@@ -25,6 +25,11 @@ const ASFGuidHash asf_payload_ext_guids[] = {
   {ASF_PAYLOAD_EXTENSION_DURATION, "ASF_PAYLOAD_EXTENSION_DURATION",
         {0xC6BD9450, 0x4907867F, 0x79C7A383, 0xAD33B721}
       },
+  {ASF_PAYLOAD_EXTENSION_SYSTEM_CONTENT, "ASF_PAYLOAD_EXTENSION_SYSTEM_CONTENT",
+      {0xD590DC20, 0x436C07BC, 0xBBF3f79C, 0xDCA4F1FB}},
+  {ASF_PAYLOAD_EXTENSION_SYSTEM_PIXEL_ASPECT_RATIO,
+        "ASF_PAYLOAD_EXTENSION_SYSTEM_PIXEL_ASPECT_RATIO",
+      {0x1b1ee554, 0x4bc8f9ea, 0x6b371a82, 0xb8c4e474}},
   {ASF_PAYLOAD_EXTENSION_UNDEFINED, "ASF_PAYLOAD_EXTENSION_UNDEFINED",
         {0, 0, 0, 0}
       }
