@@ -331,6 +331,7 @@ gst_mpeg_demux_init_stream (GstMPEGDemux * mpeg_demux,
   str->last_flow = GST_FLOW_OK;
   str->buffers_sent = 0;
   str->tags = NULL;
+  str->caps = NULL;
 }
 
 static GstMPEGStream *
@@ -358,21 +359,23 @@ gst_mpeg_demux_get_video_stream (GstMPEGDemux * mpeg_demux,
         CLASS (mpeg_demux)->video_template);
     g_free (name);
 
-    mpeg_demux->video_stream[stream_nr] = str;
-
     set_caps = TRUE;
   } else {
     /* This stream may have been created by a derived class, reset the
        size. */
     video_str = g_renew (GstMPEGVideoStream, str, 1);
-    mpeg_demux->video_stream[stream_nr] = str = (GstMPEGStream *) video_str;
+    str = (GstMPEGStream *) video_str;
   }
 
+  mpeg_demux->video_stream[stream_nr] = str;
+
   if (set_caps || video_str->mpeg_version != mpeg_version) {
     gchar *codec;
     GstTagList *list;
 
     /* We need to set new caps for this pad. */
+    if (str->caps)
+      gst_caps_unref (str->caps);
     str->caps = gst_caps_new_simple ("video/mpeg",
         "mpegversion", G_TYPE_INT, mpeg_version,
         "systemstream", G_TYPE_BOOLEAN, FALSE, NULL);
@@ -416,6 +419,7 @@ gst_mpeg_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
       type < GST_MPEG_DEMUX_AUDIO_LAST, NULL);
 
   str = mpeg_demux->audio_stream[stream_nr];
+
   if (str && str->type != type) {
     gst_element_remove_pad (GST_ELEMENT (mpeg_demux), str->pad);
     g_free (str);
@@ -430,8 +434,6 @@ gst_mpeg_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
         CLASS (mpeg_demux)->audio_template);
     g_free (name);
 
-    mpeg_demux->audio_stream[stream_nr] = str;
-
     /* new pad, set caps */
     set_caps = TRUE;
   } else {
@@ -440,10 +442,14 @@ gst_mpeg_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
     str = g_renew (GstMPEGStream, str, 1);
   }
 
+  mpeg_demux->audio_stream[stream_nr] = str;
+
   if (set_caps) {
     GstTagList *list;
 
     /* We need to set new caps for this pad. */
+    if (str->caps)
+      gst_caps_unref (str->caps);
     str->caps = gst_caps_new_simple ("audio/mpeg",
         "mpegversion", G_TYPE_INT, 1, NULL);
     if (!gst_pad_set_caps (str->pad, str->caps)) {
@@ -498,11 +504,9 @@ static gboolean
 gst_mpeg_demux_parse_packhead (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
 {
   GstMPEGDemux *demux = GST_MPEG_DEMUX (mpeg_parse);
-  guint8 *buf;
 
   parent_class->parse_packhead (mpeg_parse, buffer);
 
-  buf = GST_BUFFER_DATA (buffer);
   /* do something useful here */
 
   if (demux->pending_tags) {
@@ -647,12 +651,12 @@ gst_mpeg_demux_parse_packet (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
   guint16 datalen;
 
   GstMPEGStream *outstream = NULL;
-  guint8 *buf, *basebuf;
+  guint8 *buf;
   gint64 timestamp;
 
   GstFlowReturn ret = GST_FLOW_OK;
 
-  basebuf = buf = GST_BUFFER_DATA (buffer);
+  buf = GST_BUFFER_DATA (buffer);
   id = *(buf + 3);
   buf += 4;
 
@@ -1066,7 +1070,6 @@ gst_mpeg_demux_send_subbuffer (GstMPEGDemux * mpeg_demux,
     GstClockTimeDiff diff;
     guint64 update_time;
 
-    update_time = MIN (timestamp, mpeg_parse->current_segment.stop);
     update_time = MAX (timestamp, mpeg_parse->current_segment.start);
     diff = GST_CLOCK_DIFF (mpeg_parse->current_segment.last_stop, update_time);
     if (diff > GST_SECOND * 2) {
@@ -1333,6 +1336,8 @@ gst_mpeg_demux_reset (GstMPEGDemux * mpeg_demux)
           gst_event_new_eos ());
       gst_element_remove_pad (GST_ELEMENT (mpeg_demux),
           mpeg_demux->video_stream[i]->pad);
+      if (mpeg_demux->video_stream[i]->caps)
+        gst_caps_unref (mpeg_demux->video_stream[i]->caps);
       g_free (mpeg_demux->video_stream[i]);
       mpeg_demux->video_stream[i] = NULL;
     }
@@ -1344,6 +1349,8 @@ gst_mpeg_demux_reset (GstMPEGDemux * mpeg_demux)
           mpeg_demux->audio_stream[i]->pad);
       if (mpeg_demux->audio_stream[i]->tags)
         gst_tag_list_free (mpeg_demux->audio_stream[i]->tags);
+      if (mpeg_demux->audio_stream[i]->caps)
+        gst_caps_unref (mpeg_demux->audio_stream[i]->caps);
       g_free (mpeg_demux->audio_stream[i]);
       mpeg_demux->audio_stream[i] = NULL;
     }
@@ -1353,6 +1360,8 @@ gst_mpeg_demux_reset (GstMPEGDemux * mpeg_demux)
           gst_event_new_eos ());
       gst_element_remove_pad (GST_ELEMENT (mpeg_demux),
           mpeg_demux->private_stream[i]->pad);
+      if (mpeg_demux->private_stream[i]->caps)
+        gst_caps_unref (mpeg_demux->private_stream[i]->caps);
       g_free (mpeg_demux->private_stream[i]);
       mpeg_demux->private_stream[i] = NULL;
     }
